/*
 * IIIMF-SKK, Japanese Language Engine for 
 *                        IIIMF (Internet/Intranet Input Method Framework)
 * 
 * Copyright (C) 2002 Motonobu Ichimura <famao@kondara.org>
 *
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, and/or sell copies of the Software, and to permit persons
 * to whom the Software is furnished to do so, provided that the above
 * copyright notice(s) and this permission notice appear in all copies of
 * the Software and that both the above copyright notice(s) and this
 * permission notice appear in supporting documentation.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT
 * OF THIRD PARTY RIGHTS. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
 * HOLDERS INCLUDED IN THIS NOTICE BE LIABLE FOR ANY CLAIM, OR ANY SPECIAL
 * INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Except as contained in this notice, the name of a copyright holder
 * shall not be used in advertising or otherwise to promote the sale, use
 * or other dealings in this Software without prior written authorization
 * of the copyright holder.
 *
 */

/* $Id: palette.c,v 1.5 2002/03/27 08:57:42 famao Exp $ */

/* vi:set ts=4 sw=4: */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gdk/gdkkeysyms.h>
#include <gdk/gdkx.h>
#include <gtk/gtk.h>
#include <locale.h>
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xmd.h>

#include "xaux_common.h"
#include "xaux_ext_common.h"
#include "skk_xaux.h"

#include "pixmaps/hira.xpm"
#include "pixmaps/kata.xpm"
#include "pixmaps/jisx.xpm"
#include "pixmaps/ascii.xpm"
#include "pixmaps/code.xpm"

#include "codetable.h"
#include "support.h"

xaux_class_t xaux_class = {
	"org.kondara.skk.PaletteAux", 1,
	 NULL, (Window) 0, (Window) 0,
	(Atom) 0, (Atom) 0, (Atom) 0,
	{(Atom) 0,}, 0, {(Atom) 0,} , 
	0, NULL,
};

static GtkWidget *toplevel = NULL;
static GtkWidget *status_pixmaps[5] = { NULL, NULL, NULL, NULL, NULL};
static xaux_class_t *xc_ = 0;
static int im_ = 0;
static int ic_ = 0;

/* Position */
static void palette_reposition (GtkWidget *widget);

/* Event */
static GdkFilterReturn ext_event_handler (GdkXEvent *gdk_xevent, GdkEvent *event, gpointer user_data);
static void status_changed (gchar *text, gint type);
void skk_aux_palette_send_engine (gint int_count, gint *integers, gint str_count, gchar **strings);
static void set_menu (gint type);

/* Menu Detach */
static void menu_detach_func (GtkWidget *attach_widget, GtkMenu *menu);

/* Signal */
static void code_button_clicked (GtkWidget *widget, gpointer user_data);
static void status_button_clicked (GtkWidget *widget, gpointer user_data);
static void status_menu_activate (GtkWidget *widget, gpointer user_data);

static void show_codetable (void);
static GtkWidget *palette_new (void);
static GtkWidget *status_menu_new (void);
static GtkWidget *pixmap_new (GtkWidget *parent, gint type);

static void
menu_detach_func (GtkWidget *attach_widget, GtkMenu *menu)
{
	/* nothing to do */
	return;
}

static GtkWidget*
pixmap_new_with_xpm (GtkWidget *parent, gchar **xpm)
{
	GdkColormap *colormap;
	GdkPixmap *gdkpixmap;
	GdkBitmap *mask;
	GtkWidget *pixmap;

	colormap = gtk_widget_get_colormap (parent);
	gdkpixmap = gdk_pixmap_colormap_create_from_xpm_d (NULL, colormap, &mask,
			NULL, xpm);
	if (gdkpixmap == NULL)
		return NULL;
	pixmap = gtk_pixmap_new (gdkpixmap, mask);
	gdk_pixmap_unref (gdkpixmap);
	gdk_bitmap_unref (mask);
	return pixmap;
}

static GtkWidget* 
pixmap_new (GtkWidget *parent, gint type)
{
	GdkColormap *colormap;
	GdkPixmap *gdkpixmap;
	GdkBitmap *mask;
	GtkWidget *pixmap;
	gchar **xpm = NULL;

	colormap = gtk_widget_get_colormap (parent);
	switch (type) {
		case 0:
		case 2:
			xpm = hira_xpm;
			break;
		case 1:
			xpm = kata_xpm;
			break;
		case 3:
			xpm = ascii_xpm;
			break;
		case 4:
			xpm = jisx_xpm;
			break;
	}
	gdkpixmap = gdk_pixmap_colormap_create_from_xpm_d (NULL, colormap, &mask,
			NULL, xpm);
	if (gdkpixmap == NULL)
		return NULL;
	pixmap = gtk_pixmap_new (gdkpixmap, mask);
	gdk_pixmap_unref (gdkpixmap);
	gdk_bitmap_unref (mask);
	return pixmap;
}

static GtkWidget *
create_palette_button (gchar *text, GtkPixmap *icon)
{
	GtkWidget *button;
	GtkWidget *label;
	GtkWidget *hbox;

	button = gtk_button_new ();
	hbox = gtk_hbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER (button), hbox);
	gtk_widget_show (hbox);
	if (text) {
		label = gtk_label_new (text);
		gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);
		gtk_object_set_data (GTK_OBJECT (button), "label", (gpointer) label);
		gtk_widget_show (label);
	}
	if (icon) {
		gtk_box_pack_end (GTK_BOX (hbox), GTK_WIDGET (icon), FALSE, FALSE, 0);
		gtk_object_set_data (GTK_OBJECT (button), "icon", (gpointer) icon);
		gtk_widget_show (GTK_WIDGET (icon));
	}
	return button;
}

static GtkWidget *
palette_new (void)
{
	GtkWidget *palette;
	GtkWidget *handlebox;
	GtkWidget *toolbar;
	GtkWidget *icon;
	GtkWidget *button;

	palette = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_widget_realize (palette);
	gtk_window_set_policy (GTK_WINDOW (palette), FALSE, FALSE, TRUE);
	handlebox = gtk_handle_box_new ();
	gtk_widget_show (handlebox);
	gtk_container_add (GTK_CONTAINER (palette), handlebox);
	toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL,
			GTK_TOOLBAR_ICONS);
	gtk_widget_show (toolbar);
	gtk_container_add (GTK_CONTAINER (handlebox), toolbar);
	icon = pixmap_new (palette, 0);

	button = create_palette_button ("SKK", GTK_PIXMAP (icon));
	gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
			GTK_TOOLBAR_CHILD_WIDGET, button, NULL, NULL, NULL, NULL, NULL, NULL);
	gtk_button_set_relief (GTK_BUTTON (button), GTK_TOOLBAR (toolbar)->relief);
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC (status_button_clicked), NULL);
	gtk_object_set_data (GTK_OBJECT (palette), "status_button",
			(gpointer) button);
	gtk_widget_set_parent (button, toolbar);
	gtk_widget_queue_resize (button);
	gtk_widget_queue_resize (toolbar);
	gtk_widget_show (button);

	gtk_object_set_data (GTK_OBJECT (palette), "toolbar", (gpointer) toolbar);

	icon = pixmap_new_with_xpm (palette, code_xpm);
	button = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar),
			GTK_TOOLBAR_CHILD_BUTTON, NULL, NULL, "Code Table", NULL, icon, 
			GTK_SIGNAL_FUNC (code_button_clicked), NULL);

	return palette;
}

static void
palette_reposition (GtkWidget *widget)
{
	GtkRequisition sizereq;
	gint width,height;
	if (!GTK_WIDGET_REALIZED (widget))
		return;
	if (!GTK_IS_WINDOW (widget))
		return;
	width = gdk_screen_width ();
	height = gdk_screen_height ();
	gtk_widget_size_request (widget, &sizereq);
	gtk_window_reposition (GTK_WINDOW (toplevel), width - sizereq.width,
			height - sizereq.height);
	return;
}

static GdkFilterReturn
ext_event_handler (GdkXEvent *gdk_xevent, GdkEvent *event, gpointer user_data)
{
    XEvent *xevent;
    xevent = (XEvent *) gdk_xevent;
    if (xevent->type == ClientMessage) {
	    xaux_ext_process_client_message(GDK_DISPLAY(), (XClientMessageEvent *) xevent);
	    return GDK_FILTER_REMOVE;
    } 
	else if (xevent->type == KeyRelease) {
	 //   key_event_handler(0, 0, xevent, 0);
		    return GDK_FILTER_REMOVE;
    }
    return GDK_FILTER_CONTINUE;
}

static void
status_menu_activate (GtkWidget *widget, gpointer user_data)
{
	gint send_status[1];
	send_status[0] = GPOINTER_TO_INT (user_data);
	skk_aux_palette_send_engine (1, send_status, 0, NULL);
}

static void
status_button_clicked (GtkWidget *widget, gpointer user_data)
{
	GtkWidget *menu;
	menu = gtk_object_get_data (GTK_OBJECT (toplevel), "menu");
	if (!menu)
		return;
#if 0
	gtk_menu_popup ( GTK_MENU (menu), NULL, NULL, NULL, widget, 0, 0);
#else
	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, toplevel, 0, 0);
#endif
	return;
}

static void
code_button_clicked (GtkWidget *widget, gpointer user_data)
{
	GtkWidget *code;
	code = gtk_object_get_data (GTK_OBJECT (toplevel), "code");
	if (!code)
		return;
	gtk_widget_show (code);
	return;
}

static void
set_menu (gint type)
{
	GtkWidget *menu;
	GtkWidget *menuitem;
	menu = (GtkWidget *)gtk_object_get_data (GTK_OBJECT (toplevel), "menu");
	if (!menu)
		return;
	switch (type) {
		case SKK_MODE_JMODE_HIRA:
			menuitem = (GtkWidget *)gtk_object_get_data (GTK_OBJECT (menu), "hiragana");
			break;
		case SKK_MODE_JMODE_KATA:
			menuitem = (GtkWidget *)gtk_object_get_data (GTK_OBJECT (menu), "katakana");
			break;
		case SKK_MODE_ABBREV:
			menuitem = (GtkWidget *)gtk_object_get_data (GTK_OBJECT (menu), "abbrev");
			break;
		case SKK_MODE_LATIN:
			menuitem = (GtkWidget *)gtk_object_get_data (GTK_OBJECT (menu), "latin");
			break;
		case SKK_MODE_JISX0208_LATIN:
			menuitem = (GtkWidget *)gtk_object_get_data (GTK_OBJECT (menu), "jisx0208_latin");
			break;
		default:
			menuitem = NULL;
			break;
	}
	if (!menuitem)
		return;
#if 0
	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menuitem), TRUE);
#endif
	return;

}
static void
status_changed (gchar *text, gint type)
{
	GtkWidget *toolbar;
	GtkWidget *item;
	GtkToolbarChild *child;
	GdkBitmap *mask;
	GdkPixmap *pix;
	if (!text)
		return;

	toolbar = gtk_object_get_data (GTK_OBJECT (toplevel), "toolbar");
	if (!toolbar)
		return;
#if 0
	child = (GtkToolbarChild *) g_list_nth_data (GTK_TOOLBAR (toolbar)->children, 1);
	if (!child)
		return;
	gtk_label_set_text (GTK_LABEL (child->label), text);
#endif
	child = (GtkToolbarChild *) g_list_first (GTK_TOOLBAR (toolbar)->children)->data;
	if (!child)
		return;
	/* Icon */
	if (type < 0 || type > 4)
		return;
	if (child->widget) {
		/* label */
		item = (GtkWidget *)gtk_object_get_data (GTK_OBJECT (child->widget), "label");
		if (item) {
			gtk_label_set_text (GTK_LABEL (item), text);
		}
		item = (GtkWidget *)gtk_object_get_data (GTK_OBJECT (child->widget), "icon");
		if (item) {
			if (status_pixmaps[type] == NULL) {
				status_pixmaps[type] = pixmap_new (toplevel, type);
			}
			gtk_pixmap_get (GTK_PIXMAP (status_pixmaps[type]), &pix, &mask);
			gtk_pixmap_set (GTK_PIXMAP (item), pix, mask);
		}
	}
	/* RePositioning */
	palette_reposition (toplevel);
	return;
}

static void
show_codetable (void)
{
	GtkWidget *code;
	code = (GtkWidget *) gtk_object_get_data (GTK_OBJECT (toplevel), "code");
	if (!code)
		return;
	gtk_widget_show (code);
}

static void
reply_initialize (void)
{
	gint status[1];
	status[0] = SKK_AUX_PALETTE_OK;
	skk_aux_palette_send_engine (1, status, 0, NULL);
	return;
}

Bool
xaux_ext_Start (xaux_class_t *xc, aux_ext_data_t *aux_ext_data)
{
	gtk_widget_show (toplevel);
	gdk_window_raise (toplevel->window);
	return True;
}

Bool
xaux_ext_Done (xaux_class_t *xc, aux_ext_data_t *aux_ext_data)
{
	return True;
}

Bool
xaux_ext_Draw (xaux_class_t *xc, aux_ext_data_t *aux_ext_data)
{
	gint i;

	im_ = aux_ext_data->im;
	ic_ = aux_ext_data->ic;
	xc_ = xc;

	g_message ("fuga %d\n", aux_ext_data->integer_count);
	for (i = 0; i < aux_ext_data->string_count; i++) {
		printf ("\tsv[%02d]=%s\n",i, aux_ext_data->string_list[i].ptr);
	}
	for (i = 0; i < aux_ext_data->integer_count; i++) {
		switch (aux_ext_data->integer_list[i]) {
			/* Initialize Request */
			case SKK_AUX_PALETTE_INIT:
				reply_initialize ();
				break;
			case SKK_AUX_PALETTE_CODE:
				show_codetable ();
				break;
			/* Mode */
			case SKK_MODE_JMODE_HIRA:
				status_changed (aux_ext_data->string_list[i].ptr, 0);
				set_menu (SKK_MODE_JMODE_HIRA);
				break;
			case SKK_MODE_JMODE_KATA:
				status_changed (aux_ext_data->string_list[i].ptr, 1);
				set_menu (SKK_MODE_JMODE_KATA);
				break;
			case SKK_MODE_ABBREV:
				status_changed (aux_ext_data->string_list[i].ptr, 2);
				set_menu (SKK_MODE_ABBREV);
				break;
			case SKK_MODE_LATIN:
				status_changed (aux_ext_data->string_list[i].ptr, 3);
				set_menu (SKK_MODE_LATIN);
				break;
			case SKK_MODE_OFF:
#if 1
				status_changed (aux_ext_data->string_list[i].ptr, 3);
#else
				/* TODO use SkkConf */
				gtk_widget_hide (toplevel);
#endif				
				break;
			case SKK_MODE_JISX0208_LATIN:
				status_changed (aux_ext_data->string_list[i].ptr, 4);
				set_menu (SKK_MODE_JISX0208_LATIN);
				break;
			default:
				break;
		}
	}
	return True;
}


static GtkWidget *
status_menu_new (void)
{
	GtkWidget *menu;
	GtkAccelGroup *accel;
	GSList *status_group = NULL;
	GtkWidget *hiragana;
	GtkWidget *katakana;
	GtkWidget *jisx0208_latin;
	GtkWidget *latin;
	GtkWidget *abbrev;

	menu = gtk_menu_new ();
	accel = gtk_menu_ensure_uline_accel_group (GTK_MENU (menu));

	hiragana = gtk_radio_menu_item_new_with_label
		(status_group, (_("hiragana")));
	gtk_object_set_data (GTK_OBJECT (hiragana), "status", GINT_TO_POINTER (SKK_MODE_JMODE_HIRA));
	gtk_object_set_data (GTK_OBJECT (menu), "hiragana", (gpointer) hiragana);
	status_group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (hiragana));

	katakana = gtk_radio_menu_item_new_with_label
		(status_group, (_("katakana")));
	gtk_object_set_data (GTK_OBJECT (katakana), "status", GINT_TO_POINTER (SKK_MODE_JMODE_KATA));
	gtk_object_set_data (GTK_OBJECT (menu), "katakana", (gpointer) katakana);
	status_group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (katakana));

	jisx0208_latin = gtk_radio_menu_item_new_with_label
		(status_group, (_("jisx0208_latin")));
	gtk_object_set_data (GTK_OBJECT (jisx0208_latin), "status", GINT_TO_POINTER (SKK_MODE_JISX0208_LATIN));
	gtk_object_set_data (GTK_OBJECT (menu), "jisx0208_latin", (gpointer) jisx0208_latin);
	status_group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (jisx0208_latin));

	latin = gtk_radio_menu_item_new_with_label
		(status_group, (_("latin")));
	gtk_object_set_data (GTK_OBJECT (latin), "status", GINT_TO_POINTER (SKK_MODE_LATIN));
	gtk_object_set_data (GTK_OBJECT (menu), "latin", (gpointer) latin);
	status_group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (latin));

	abbrev = gtk_radio_menu_item_new_with_label
		(status_group, (_("abbrev")));
	gtk_object_set_data (GTK_OBJECT (abbrev), "status", GINT_TO_POINTER (SKK_MODE_ABBREV));
	gtk_object_set_data (GTK_OBJECT (menu), "abbrev", (gpointer) abbrev);
	status_group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (abbrev));

	gtk_container_add (GTK_CONTAINER (menu), hiragana);
	gtk_container_add (GTK_CONTAINER (menu), katakana);
	gtk_container_add (GTK_CONTAINER (menu), jisx0208_latin);
	gtk_container_add (GTK_CONTAINER (menu), latin);
	gtk_container_add (GTK_CONTAINER (menu), abbrev);

	gtk_signal_connect (GTK_OBJECT (hiragana), "activate", 
			GTK_SIGNAL_FUNC (status_menu_activate), GINT_TO_POINTER (SKK_MODE_JMODE_HIRA));
	gtk_signal_connect (GTK_OBJECT (katakana), "activate",
			GTK_SIGNAL_FUNC (status_menu_activate), GINT_TO_POINTER (SKK_MODE_JMODE_KATA));
	gtk_signal_connect (GTK_OBJECT (jisx0208_latin), "activate",
			GTK_SIGNAL_FUNC (status_menu_activate), GINT_TO_POINTER (SKK_MODE_JISX0208_LATIN));
	gtk_signal_connect (GTK_OBJECT (latin), "activate",
			GTK_SIGNAL_FUNC (status_menu_activate), GINT_TO_POINTER (SKK_MODE_LATIN));
	gtk_signal_connect (GTK_OBJECT (abbrev), "activate",
			GTK_SIGNAL_FUNC (status_menu_activate), GINT_TO_POINTER (SKK_MODE_ABBREV));
	gtk_widget_show (hiragana);
	gtk_widget_show (katakana);
	gtk_widget_show (jisx0208_latin);
	gtk_widget_show (latin);
	gtk_widget_show (abbrev);
	return menu;
}

void
skk_aux_palette_send_engine (gint int_count, gint *integers, gint str_count, gchar **strings) 
{
	gint i;
	aux_ext_data_t *aux_data;

	if (!im_ || !ic_ || !xc_)
		return;
	aux_data = (aux_ext_data_t *) g_new0 (aux_ext_data_t,1);
	aux_data->im = im_;
	aux_data->ic = ic_;
	aux_data->integer_count = int_count;
	aux_data->string_count = str_count;

	if (aux_data->integer_count) {
		aux_data->integer_list = (int *) g_new0 (int, aux_data->integer_count);
		for (i = 0; i < aux_data->integer_count; i++) {
			aux_data->integer_list[i] = integers[i];
		}
	}
	if (aux_data->string_count) {
		aux_data->string_list = (aux_ext_string_t *) g_new0 (aux_ext_string_t, aux_data->string_count);
		for (i = 0; i < aux_data->string_count; i++) {
			aux_ext_string_t *p = &aux_data->string_list[i];
			p->length = strlen (strings[i]);
			p->ptr = (guchar *) g_strdup (strings[i]);
		}
	}
	xaux_ext_SetValue (GDK_DISPLAY(), xc_, aux_data);

	if (aux_data->integer_list) {
		g_free (aux_data->integer_list);
	}
	if (aux_data->string_list) {
		for (i = 0; i < aux_data->string_count; i++) {
			aux_ext_string_t *p = &aux_data->string_list[i];
			if (p && p->ptr)
				g_free (p->ptr);
		}
		g_free (aux_data->string_list);
	}
	g_free (aux_data);
	return;
}

int
main (int argc, char *argv[])
{
	GtkWidget *menu;
	GtkWidget *code;
	/* force to EUC-JP */
	setenv ("LC_ALL","ja_JP.eucJP",True);

	gtk_set_locale ();

#ifdef ENABLE_NLS
	bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
	textdomain (PACKAGE);
#endif

	gtk_init (&argc, &argv);
	toplevel = palette_new ();
	gdk_window_set_decorations (toplevel->window,0);
	gdk_window_set_override_redirect (toplevel->window, FALSE);
	gtk_widget_show (toplevel);
	palette_reposition (toplevel);
	menu = status_menu_new ();
	gtk_menu_attach_to_widget (GTK_MENU (menu), toplevel, menu_detach_func);
	gtk_object_set_data (GTK_OBJECT (toplevel), "menu", (gpointer)menu);
	code = skk_aux_code_new ();
	gtk_object_set_data (GTK_OBJECT (toplevel), "code", (gpointer)code);

	gdk_window_add_filter (toplevel->window, ext_event_handler, 0);
	gtk_widget_hide (toplevel);
	if (xaux_ext_init_classes (GDK_DISPLAY (),
				&xaux_class, GDK_WINDOW_XWINDOW (toplevel->window)) == False) {
		exit (1);
	}
	gtk_main ();
	return 0;
}
