#include <string.h>
#include "skkquery.h"
#include "skkutils.h"
#include "skkldict.h"
#include "skkdict.h"

SkkQuery *
skk_query_new (void)
{
	SkkQuery *ret;
	ret = g_new (SkkQuery, 1);
	memset (ret, 0, sizeof (SkkQuery));
	ret->query_list = NULL;
	ret->ref_count++;
	ret->cur_item = NULL;
	return ret;
}

SkkQueryItem *
skk_query_item_new (gchar *filename, SkkQueryType type)
{
	SkkQueryItem *item;
	item = g_new (SkkQueryItem, 1);
	memset (item, 0, sizeof (SkkQueryItem));
	item->file = filename;
	item->type = type;
	return item;
}

void
skk_query_destroy (SkkQuery *query)
{
	if (!query)
		return;
	query->ref_count--;
	if (query->ref_count > 0)
		return;
	if (query->query_list) {
		/* FIXME maybe memory leaks */
		skk_utils_list_free (query->query_list, TRUE);
		query->query_list = NULL;
	}
	g_free (query);
	return;
}

void
skk_query_add_item (SkkQuery *query, SkkQueryItem *item)
{
	if (!query)
		return;
	if (!item)
		return;
	query->query_list = g_list_append (query->query_list, (gpointer)item);
	return;
}

gboolean
skk_query_set_next (SkkQuery *query)
{
	if (!query)
		return FALSE;
	query->cur_list_num++;
	query->cur_item = g_list_nth_data (query->query_list, query->cur_list_num);
	if (query->cur_item)
		return TRUE;
	return FALSE;
}

gboolean
skk_query_set_nth (SkkQuery *query, gint nth)
{
	SkkQueryItem *tmp_item;
	if (!query)
		return FALSE;
	tmp_item = query->cur_item;	
	query->cur_item = (SkkQueryItem *)g_list_nth_data (query->query_list, nth);
	if (query->cur_item) {
		query->cur_list_num = nth;
		return TRUE;
	}
	query->cur_item = tmp_item;
	return FALSE;
}	

gboolean
skk_query_set_prev (SkkQuery *query)
{
	if (!query)
		return FALSE;
	if (query->cur_list_num == 0)
		return FALSE;
	query->cur_list_num--;
	query->cur_item = g_list_nth_data (query->query_list, query->cur_list_num);
	if (query->cur_item)
		return TRUE;
	return FALSE;
}

GList *
skk_query_do_query (SkkQuery *query, gchar *buf, gchar *okuri, gint *found_num)
{
	if (!query)
		return NULL;
	if (!query->cur_item) {
		query->cur_item = (SkkQueryItem *)g_list_nth_data (query->query_list, 0);
	}
	if (!query->cur_item)
		return NULL;
	switch (query->cur_item->type) {
		case SKKQUERY_LOCAL:
			return skk_ldict_query_string (buf, okuri, found_num);
			break;
		case SKKQUERY_SERVER:
			return skk_dict_query_string (buf, okuri, found_num);
			break;
		default:
			break;
	}
	return NULL;
}
